import logging

from google.cloud import vision
from google.cloud.vision import types
from google.oauth2 import service_account

log: logging.Logger = logging.getLogger(__name__)


class Image:
    def __init__(self, service_file, image_content=None, image_uri=None) -> None:
        self.google = vision.ImageAnnotatorClient(
            credentials=service_account.Credentials.from_service_account_file(service_file)
        )

        self.image = types.Image()
        if image_content:
            self.image.content = image_content
        elif image_uri:
            self.image.source.image_uri = image_uri
        else:
            raise AttributeError("Please define either image_uri or image_content")

    def get_text(self) -> None:
        log.info("Requesting TEXT_DETECTION from Google Cloud API. This will cost us 0.0015 USD.")
        attempts = 0
        while attempts < 5:
            attempts += 1
            response = self.google.text_detection(image=self.image)
            try:
                response.text_annotations[0]
            except IndexError:
                pass
            else:
                self.text_found = response.text_annotations
                break

        if attempts == 5:
            raise Exception(response.text_annotations)



"""This is taken from babel.numbers and modified. The following license applies


Copyright (c) 2013-2019 by the Babel Team, see AUTHORS for more information.

All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions
are met:

 1. Redistributions of source code must retain the above copyright
    notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
    notice, this list of conditions and the following disclaimer in
    the documentation and/or other materials provided with the
    distribution.
 3. The name of the author may not be used to endorse or promote
    products derived from this software without specific prior
    written permission.

THIS SOFTWARE IS PROVIDED BY THE AUTHOR "AS IS" AND ANY EXPRESS
OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
import decimal
from typing import Dict

from babel.numbers import NumberFormatError


def parse_number(string, locale: Dict[str, str] = dict(group=",", decimal=".")):
    group_symbol = locale.get("group", ",")
    if (
        group_symbol == "\xa0"
        and group_symbol not in string  # if the grouping symbol is U+00A0 NO-BREAK SPACE,
        and " "  # and the string to be parsed does not contain it,
        in string  # but it does contain a space instead,
    ):
        # ... it's reasonable to assume it is taking the place of the grouping symbol.
        string = string.replace(" ", group_symbol)

    try:
        return int(string.replace(group_symbol, ""))
    except ValueError:
        raise NumberFormatError("%r is not a valid number" % string)


def parse_decimal(string, locale: Dict[str, str] = dict(group=",", decimal=".")):
    group_symbol = locale.get("group", ",")
    decimal_symbol = locale.get("decimal", ".")

    if (
        group_symbol == "\xa0"
        and group_symbol not in string  # if the grouping symbol is U+00A0 NO-BREAK SPACE,
        and " "  # and the string to be parsed does not contain it,
        in string  # but it does contain a space instead,
    ):
        # ... it's reasonable to assume it is taking the place of the grouping symbol.
        string = string.replace(" ", group_symbol)

    try:
        parsed = decimal.Decimal(string.replace(group_symbol, "").replace(decimal_symbol, "."))
    except decimal.InvalidOperation:
        raise NumberFormatError("%r is not a valid decimal number" % string)
    return parsed



import datetime
import decimal
import json
import logging
import os
import re
from string import digits
from typing import Optional

import babel
from dateutil.parser import parse as parse_date

from .cloudvision import Image
from .numbers import parse_decimal, parse_number

log: logging.Logger = logging.getLogger(__name__)


class ProfileSelf(Image):
    def __init__(self, service_file, image_content=None, image_uri=None) -> None:
        super().__init__(service_file, image_content=image_content, image_uri=image_uri)
        self.locale = babel.Locale.parse("en")
        self.numeric_locale = {}
        with open(os.path.join(os.path.dirname(__file__), "pattern_lookups.json"), "r") as f:
            self.pattern_lookups = json.load(f)

    def get_text(self, force: Optional[bool] = False) -> None:
        # Check if an API call needs to be made.
        # Prevents accidental extra calls to a premium API
        if force or not hasattr(self, "text_found"):
            super().get_text()

        # Try to work out locale:
        locale_lookup = [
            (
                babel.Locale.parse("fr"),
                r"(?:(?:Activit[ée]s\stotales)|(?:PROGR[ÈE]S\sDE\sLA\sSEMAINE))",
            ),
            (
                babel.Locale.parse("de"),
                r"(?:(?:Aktivit[äa]tsstatistik)|(?:W[ÖO]CHENTLICHER\sFORTSCHRITT))",
            ),
            (
                babel.Locale.parse("it"),
                r"(?:(?:Riepilogo\sattivit[àa])|(?:PROGRESSI\sSETTIMANALI))",
            ),
            (babel.Locale.parse("ja"), r"(?:(?:アクティビティ)|(?:ウィークリー))"),
            (babel.Locale.parse("ko"), r"(?:(?:활동)|(?:주간\s피트니스))"),
            (babel.Locale.parse("es"), r"(?:(?:Total\sde\sactividades)|(?:PROGRESO\sSEMANAL))"),
            (babel.Locale.parse("zh_hant"), r"(?:(?:活動紀錄)|(?:本週成果))"),
            (babel.Locale.parse("en"), r"(?:(?:Total\sActivity)|(?:WEEKLY\sPROGRESS))"),
            (babel.Locale.parse("pt_br"), r"(?:(?:Total\sde\satividades)|(?:PROGRESSO\sSEMANAL))"),
            (babel.Locale.parse("th"), r"(?:(?:กิจกรรมทั้งหมด)|(?:ความคืบหน้าประจำสัปดาห์))"),
        ]
        for locale, pattern in locale_lookup:
            if re.search(pattern, self.text_found[0].description, re.IGNORECASE):
                self.locale = locale
                break

        # Since the only stat that is a decimal is travel_km, we will check that.
        try:
            travel_km = (
                re.search(
                    self.pattern_lookups[self.locale.language]["travel_km"],
                    self.text_found[0].description,
                    re.IGNORECASE,
                )[1]
                .replace("km", "")
                .strip()
                .replace(" ", "\xa0")
            )
        except (decimal.InvalidOperation, TypeError):
            pass
        else:
            translated = travel_km.translate(str.maketrans("", "", digits))
            self.numeric_locale["decimal"] = translated[-1]
            if len(translated) > 1:
                self.numeric_locale["group"] = translated[-2]

        # If travel_km wasn't big enough to get the group for numeric_locale, try total_xp
        if self.numeric_locale.get("group") is None:
            try:
                total_xp = (
                    re.search(
                        self.pattern_lookups[self.locale.language]["total_xp"],
                        self.text_found[0].description,
                        re.IGNORECASE,
                    )[1]
                    .strip()
                    .replace(" ", "\xa0")
                )
            except (ValueError, TypeError):
                # Assume Locale
                self.numeric_locale["group"] = babel.Locale.get_group_symbol(self.locale)
            else:
                translated = total_xp.translate(str.maketrans("", "", digits))
                self.numeric_locale["group"] = translated[0]

    @property
    def username(self) -> Optional[str]:
        try:
            return re.search(
                r"([A-Za-z0-9]+)\n(?:\&|et|con|e) ?(.+)", self.text_found[0].description
            )[1]
        except TypeError:
            log.exception("username failed to return")
            return None

    @property
    def buddy_name(self) -> Optional[str]:
        try:
            return re.search(
                r"([A-Za-z0-9]+)\n(?:\&|et|con|e) ?(.+)", self.text_found[0].description
            )[2]
        except TypeError:
            log.exception("buddy_name failed to return")
            return None

    @property
    def travel_km(self) -> Optional[decimal.Decimal]:
        try:
            result = (
                re.search(
                    self.pattern_lookups[self.locale.language]["travel_km"],
                    self.text_found[0].description,
                    re.IGNORECASE,
                )[1]
                .replace("km", "")
                .strip()
            )
            return parse_decimal(result, locale=self.numeric_locale)
        except (babel.numbers.NumberFormatError, TypeError):
            log.exception("travel_km failed to return")
            return None

    @property
    def capture_total(self) -> Optional[int]:
        try:
            result = re.search(
                self.pattern_lookups[self.locale.language]["capture_total"],
                self.text_found[0].description,
                re.IGNORECASE,
            )[1].strip()
            return parse_number(result, locale=self.numeric_locale)
        except (babel.numbers.NumberFormatError, TypeError):
            log.exception("capture_total failed to return")
            return None

    @property
    def pokestops_visited(self) -> Optional[int]:
        try:
            result = re.search(
                self.pattern_lookups[self.locale.language]["pokestops_visited"],
                self.text_found[0].description,
                re.IGNORECASE,
            )[1].strip()
            return parse_number(result, locale=self.numeric_locale)
        except (babel.numbers.NumberFormatError, TypeError):
            log.exception("pokestops_visited failed to return")
            return None

    @property
    def total_xp(self) -> Optional[int]:
        try:
            result = re.search(
                self.pattern_lookups[self.locale.language]["total_xp"],
                self.text_found[0].description,
                re.IGNORECASE,
            )[1].strip()
            return parse_number(result, locale=self.numeric_locale)
        except (babel.numbers.NumberFormatError, TypeError):
            log.exception("total_xp failed to return")
            return None

    @property
    def start_date(self) -> Optional[datetime.date]:
        try:
            result = re.search(
                self.pattern_lookups[self.locale.language]["start_date"],
                self.text_found[0].description,
                re.IGNORECASE,
            )[1]
            return parse_date(result).date()
        except (ValueError, OverflowError, TypeError):
            log.exception("start_date failed to return")
            return None

    def find_stats(self):
        return self.get_text(force=False)
